@@ -1,6 +1,14 @@
 import re
 import inspect
 import os
+import sys
+
+import six
+
+if sys.version_info >= (3, 3):
+    from importlib.machinery import SourceFileLoader
+else:
+    import imp
 
 
 IDENTIFIER = re.compile(r'[a-z_](\w)*$', re.IGNORECASE)
@@ -56,7 +64,7 @@ class Config(object):
         '''
 
         if isinstance(conf_dict, dict):
-            iterator = conf_dict.iteritems()
+            iterator = six.iteritems(conf_dict)
         else:
             iterator = iter(conf_dict)
 
@@ -81,7 +89,7 @@ class Config(object):
         '''
         Private helper method for to_dict.
         '''
-        for k, v in obj.items():
+        for k, v in obj.copy().items():
             if prefix:
                 del obj[k]
                 k = "%s%s" % (prefix, k)
@@ -118,21 +126,21 @@ class Config(object):
                 self.__values__[key] = ConfigDict(value)
             else:
                 self.__values__[key] = Config(value, filename=self.__file__)
-        elif isinstance(value, basestring) and '%(confdir)s' in value:
+        elif isinstance(value, six.string_types) and '%(confdir)s' in value:
             confdir = os.path.dirname(self.__file__) or os.getcwd()
             self.__values__[key] = value.replace('%(confdir)s', confdir)
         else:
             self.__values__[key] = value
 
     def __iter__(self):
-        return self.__values__.iteritems()
+        return six.iteritems(self.__values__)
 
     def __dir__(self):
         """
         When using dir() returns a list of the values in the config.  Note:
         This function only works in Python2.6 or later.
         """
-        return self.__values__.keys()
+        return list(self.__values__.keys())
 
     def __repr__(self):
         return 'Config(%s)' % str(self.__values__)
@@ -150,7 +158,24 @@ def conf_from_file(filepath):
     if not os.path.isfile(abspath):
         raise RuntimeError('`%s` is not a file.' % abspath)
 
-    execfile(abspath, globals(), conf_dict)
+    # First, make sure the code will actually compile (and has no SyntaxErrors)
+    with open(abspath, 'rb') as f:
+        compiled = compile(f.read(), abspath, 'exec')
+
+    # Next, attempt to actually import the file as a module.
+    # This provides more verbose import-related error reporting than exec()
+    absname, _ = os.path.splitext(abspath)
+    basepath, module_name = absname.rsplit(os.sep, 1)
+    if sys.version_info >= (3, 3):
+        SourceFileLoader(module_name, abspath).load_module(module_name)
+    else:
+        imp.load_module(
+            module_name,
+            *imp.find_module(module_name, [basepath])
+        )
+
+    # If we were able to import as a module, actually exec the compiled code
+    exec(compiled, globals(), conf_dict)
     conf_dict['__file__'] = abspath
 
     return conf_from_dict(conf_dict)
@@ -182,7 +207,7 @@ def conf_from_dict(conf_dict):
     '''
     conf = Config(filename=conf_dict.get('__file__', ''))
 
-    for k, v in conf_dict.iteritems():
+    for k, v in six.iteritems(conf_dict):
         if k.startswith('__'):
             continue
         elif inspect.ismodule(v):
@@ -215,7 +240,7 @@ def set_config(config, overwrite=False):
     if overwrite is True:
         _runtime_conf.empty()
 
-    if isinstance(config, basestring):
+    if isinstance(config, six.string_types):
         config = conf_from_file(config)
         _runtime_conf.update(config)
         if config.__file__:
