@@ -48,7 +48,13 @@ class RoutingState(object):
 
 
 class Request(WebObRequest):
-    pass
+
+    def __getattribute__(self, name):
+        try:
+            return WebObRequest.__getattribute__(self, name)
+        except UnicodeDecodeError as e:
+            logger.exception(e)
+            abort(400)
 
 
 class Response(WebObResponse):
@@ -390,18 +396,19 @@ class PecanBase(object):
         return args, varargs, kwargs
 
     def render(self, template, namespace):
-        renderer = self.renderers.get(
-            self.default_renderer,
-            self.template_path
-        )
         if template == 'json':
             renderer = self.renderers.get('json', self.template_path)
-        if ':' in template:
+        elif ':' in template:
+            renderer_name, template = template.split(':', 1)
             renderer = self.renderers.get(
-                template.split(':')[0],
+                renderer_name,
+                self.template_path
+            )
+        else:
+            renderer = self.renderers.get(
+                self.default_renderer,
                 self.template_path
             )
-            template = template.split(':')[1]
         return renderer.render(template, namespace)
 
     def find_controller(self, state):
@@ -504,22 +511,22 @@ class PecanBase(object):
 
         # fetch any parameters
         if req.method == 'GET':
-            params = dict(req.GET)
+            params = req.GET
         elif req.content_type in ('application/json',
                                   'application/javascript'):
             try:
                 if not isinstance(req.json, dict):
                     raise TypeError('%s is not a dict' % req.json)
-                params = dict(NestedMultiDict(req.GET, req.json))
+                params = NestedMultiDict(req.GET, req.json)
             except (TypeError, ValueError):
-                params = dict(req.params)
+                params = req.params
         else:
-            params = dict(req.params)
+            params = req.params
 
         # fetch the arguments for the controller
         args, varargs, kwargs = self.get_args(
             state,
-            params,
+            params.mixed(),
             remainder,
             cfg['argspec'],
             im_self
