@@ -12,6 +12,7 @@ from webob.exc import HTTPNotFound
 from webtest import TestApp
 import six
 from six import b as b_
+from six import u as u_
 from six.moves import cStringIO as StringIO
 
 from pecan import (
@@ -145,6 +146,23 @@ class TestAppIterFile(PecanTestCase):
         assert len(r.body) == 0
 
 
+class TestInvalidURLEncoding(PecanTestCase):
+
+    @property
+    def app_(self):
+        class RootController(object):
+
+            @expose()
+            def _route(self, args, request):
+                assert request.path
+
+        return TestApp(Pecan(RootController()))
+
+    def test_rest_with_non_utf_8_body(self):
+        r = self.app_.get('/%aa/', expect_errors=True)
+        assert r.status_int == 400
+
+
 class TestIndexRouting(PecanTestCase):
 
     @property
@@ -872,6 +890,33 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('variable_all: 7, day=12, id=seven, month=1')
 
+    def test_duplicate_query_parameters_GET(self):
+        r = self.app_.get('/variable_kwargs?list=1&list=2')
+        l = [u_('1'), u_('2')]
+        assert r.status_int == 200
+        assert r.body == b_('variable_kwargs: list=%s' % l)
+
+    def test_duplicate_query_parameters_POST(self):
+        r = self.app_.post('/variable_kwargs',
+                           {'list': ['1', '2']})
+        l = [u_('1'), u_('2')]
+        assert r.status_int == 200
+        assert r.body == b_('variable_kwargs: list=%s' % l)
+
+    def test_duplicate_query_parameters_POST_mixed(self):
+        r = self.app_.post('/variable_kwargs?list=1&list=2',
+                           {'list': ['3', '4']})
+        l = [u_('1'), u_('2'), u_('3'), u_('4')]
+        assert r.status_int == 200
+        assert r.body == b_('variable_kwargs: list=%s' % l)
+
+    def test_duplicate_query_parameters_POST_mixed_json(self):
+        r = self.app_.post('/variable_kwargs?list=1&list=2',
+                           {'list': 3})
+        l = [u_('1'), u_('2'), u_('3')]
+        assert r.status_int == 200
+        assert r.body == b_('variable_kwargs: list=%s' % l)
+
     def test_no_remainder(self):
         try:
             r = self.app_.get('/eater')
